@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: AttachSettings.py 10407 2017-10-24 10:14:28Z David $
+# $Id: AttachSettings.py 10687 2017-12-26 11:58:17Z David $
 #
 # Copyright (c) 2013 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: David $
-# $Date: 2017-10-24 18:14:28 +0800 (Tue, 24 Oct 2017) $
-# $Revision: 10407 $
+# $Date: 2017-12-26 19:58:17 +0800 (Tue, 26 Dec 2017) $
+# $Revision: 10687 $
 
 import os
 
@@ -61,7 +61,9 @@ def attachSettings(settingsDict):
             'Theophrastus.statistics',
             'Theophrastus.document',
             # django-selectable
-            'selectable',         
+            'selectable',
+            # django-modeladmin-reorder
+            'admin_reorder',     
         ])
 
         if settingsDict.get('THEOPHRASTUS_ENABLE_INTEGRATION', False):
@@ -80,6 +82,16 @@ def attachSettings(settingsDict):
             'THEOPHRASTUS_ENABLE_INTEGRATION': ('ENABLE_INTEGRATION', False),
         })
 
+
+        #=======================================================================
+        # Attach editor middleware settings.
+        #=======================================================================
+
+        # django-modeladmin-reorder
+        middlewareClasses = settingsDict['MIDDLEWARE_CLASSES']
+        middlewareClasses += ('admin_reorder.middleware.ModelAdminReorder', )
+        settingsDict['MIDDLEWARE_CLASSES'] = middlewareClasses
+
         #=======================================================================
         # Attach editor template settings.
         #=======================================================================
@@ -205,6 +217,24 @@ def attachSettings(settingsDict):
 
         #=======================================================================
 
+        #=======================================================================
+        # django-modeladmin-reorder.
+        #=======================================================================
+        adminReorderDict = {
+            'THEOPHRASTUS_ADMIN_REORDER':(
+                'ADMIN_REORDER', (
+                    # Keep original label and models
+                    'auth',      
+                    'integration',        
+                    'newsletter',
+                    'newsletter_extension',
+                    'sites',
+                ),
+            ),
+        }
 
+        attachAttrs(attrs=adminReorderDict)
+        
+            
         from Iuno import Patch
         Patch.extendAll(settingsDict)
