@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: StatisticsHandler.py 10703 2018-01-02 07:14:20Z David $
+# $Id: StatisticsHandler.py 10744 2018-01-28 15:00:13Z David $
 #
 # Copyright (c) 2012 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: David $
-# $Date: 2018-01-02 15:14:20 +0800 (Tue, 02 Jan 2018) $
-# $Revision: 10703 $
+# $Date: 2018-01-28 23:00:13 +0800 (Sun, 28 Jan 2018) $
+# $Revision: 10744 $
 
 from datetime import timedelta, datetime
 
@@ -292,7 +292,7 @@ def getEmptyDataStructure():
 def getEmptyDataList():
     return []
 
-def countStatus(queryset):
+def countStatus(queryset, historyMode='general'):
     """
     Calculate the status number of occurrences.
     ex: (c.newsletter.id, c.status, c.creation_date)
@@ -384,26 +384,48 @@ def countStatus(queryset):
 #     clickedLinks = queryset.filter(status=ContactMailingStatus.LINK_OPENED)
 #     uniqueClicked = len(set(clickedLinks.values_list(
 #                               'contact', flat=True)))
-    return [{
-        'sentTest': len(sentTestNumber),
-        'deliverie': len(sentNumber),
-        'error': len(errorLogs),
-        'invaild': len(invaildLogs),
-        'open': len(openLogs),
-        'openOnSite': len(openOnSiteLogs),
-        'click': len(clickLogs),
-        'unsubscribe': len(unsubscriptionNumber),
-        'bounce': len(bouncedLogs),
-        'complain': len(complainLogs),
-        'reject': len(rejectedLogs),        
-        
-        },
-        {
-         'openId': openId,
-         'openOnSiteId': openOnSiteId,
-         'clickId': clickId
-        }            
-    ]
+    if historyMode == 'history':
+        return [{
+            'sentTest': len(sentTestNumber),
+            'deliverie': len(sentNumber),
+            'error': len(errorLogs),
+            'invaild': len(invaildLogs),
+            'open': len(openId),
+            'openOnSite': len(openOnSiteId),
+            'click': len(clickId),
+            'unsubscribe': len(unsubscriptionNumber),
+            'bounce': len(bouncedLogs),
+            'complain': len(complainLogs),
+            'reject': len(rejectedLogs),        
+            
+            },
+            {
+            'openId': openId,
+            'openOnSiteId': openOnSiteId,
+            'clickId': clickId
+            }            
+        ]
+    else:
+        return [{
+            'sentTest': len(sentTestNumber),
+            'deliverie': len(sentNumber),
+            'error': len(errorLogs),
+            'invaild': len(invaildLogs),
+            'open': len(openLogs),
+            'openOnSite': len(openOnSiteLogs),
+            'click': len(clickLogs),
+            'unsubscribe': len(unsubscriptionNumber),
+            'bounce': len(bouncedLogs),
+            'complain': len(complainLogs),
+            'reject': len(rejectedLogs),        
+            
+            },
+            {
+            'openId': openId,
+            'openOnSiteId': openOnSiteId,
+            'clickId': clickId
+            }            
+        ]
 
 from django.db.models import Q
 
@@ -451,6 +473,7 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
     
     dataType = reqestGetData.get('dataType', 'number')
     mode = reqestGetData.get('mode', 'accumulate')
+    historyMode = reqestGetData.get('historyMode', 'general')
     allDataDates = getAllDataDate()
     
 #     period = reqestGetData.get('period', 'week')
@@ -463,13 +486,17 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
 #     dates = getDateRange(period, defaultStart=_t)
     
 
-    def getContactStatus(conditions):
+    def getContactStatus(conditions, history):
         contactStatus = ContactMailingStatus.objects.filter(**conditions)
         selectData = {"d": "strftime('%%Y-%%m-%%d', "
                             "newsletter_contactmailingstatus.creation_date)"}
-        contactStatus = contactStatus.extra(select=selectData).values(
-            'contact', 'status', 'd').distinct(
-            ).order_by('contact', 'status', 'd')
+        if historyMode == 'history':
+            contactStatus = contactStatus.extra(select=selectData).values(
+                'contact', 'status', 'd').order_by('contact', 'status', 'd')
+        else:
+            contactStatus = contactStatus.extra(select=selectData).values(
+                'contact', 'status', 'd').distinct(
+                ).order_by('contact', 'status', 'd')
 
         # [{'status': 0, 'contact': 1, 'd': u'2012-12-20'},
         #  {'status': 4, 'contact': 1, 'd': u'2012-12-20'}, ...]
@@ -503,70 +530,113 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
     if 'newsletter__id__in' in conditions:
         newsletterIds = conditions.pop('newsletter__id__in')
 
-        dataTmp = {}
-        # data structure of dataTmp is something like:
-        # {[u'2017-12-30', {'bounce': 0, 'click': 0, ...}], ...}
-        for _id in newsletterIds:
-            conditions.update({'newsletter__id': _id})
-            _cs = getContactStatus(conditions)
-            _dateKey = _cs[0]['d']
-            _statusCount = countStatus(_cs)[0]
-            dataTmp[_dateKey] = _statusCount
-        # Merge.
-        totalData = []
-        for _dt in dataTmp:
-            data = {}
-            allDatas = dataTmp[_dt]
-            for _key in allDatas:
-                v = allDatas[_key]
-                if _key not in data:
-                    data[_key] = 0
-                data[_key] += v
-            totalData.append([_dt, data])
-    else:
-        contactStatus = getContactStatus(conditions)
-        completed = sortByDatetime(contactStatus)
-        totalData = []
+        
+        allDates = []
+        allNewslettersData = []
         openIds = []
         openOnSiteIds = []
-        linkIds = []
-        for c in completed:
-            _d = countStatus(completed[c])           
-            open = _d[1]['openId']
-            openOnSite = _d[1]['openOnSiteId']
-            link = _d[1]['clickId']
-            
-            openCount = 0
-            openOnSiteCount = 0
-            linkCount = 0
-            
-            for id in open:
-                if not id in openIds:
+        linkOpenIds = []
+        # data structure of allNewslettersData is something like:
+        # [ {u'2017-12-30': {'bounce': 0, 'click': 0, ...}}, ...}
+        for _id in newsletterIds:
+            conditions.update({'newsletter__id': _id})
+            _cs = getContactStatus(conditions, historyMode)
+            if _cs:
+                # _cs:[ {'status': 0, 'contact': 5, 'd': u'2018-01-04'}, 
+                # {'status': 4, 'contact': 5, 'd': u'2018-01-07'}...]
+                
+                for ele in _cs:
+                    contactId = ele['contact']
+                    _dateKey = ele['d']
+                    if _dateKey not in allDates:
+                        allDates.append(_dateKey)
+
+                    _statusCount = countStatus([ele, ], historyMode)[0]
+
+                    if _statusCount['open']:
+                        if contactId in openIds and historyMode == 'general':
+                            _statusCount['open'] = 0
+                        else:
+                            openIds.append(contactId)
+                    if _statusCount['openOnSite']:        
+                        if contactId in openOnSiteIds and historyMode == 'general':
+                            _statusCount['openOnSite'] = 0
+                        else:
+                            openIds.append(contactId)
+                    if _statusCount['click']:
+                        if contactId in openOnSiteIds and historyMode == 'general':
+                            _statusCount['click'] = 0
+                        else:
+                            openOnSiteIds.append(contactId)
+
+                    dataTmp = {}
+                    dataTmp[_dateKey] = _statusCount
+                    allNewslettersData.append(dataTmp)
+        # Merge.
+        totalData = []
+        for _dt in allDates:
+            data = {}
+            for newsletterData in allNewslettersData:
+                if _dt in newsletterData:
+                    allDatas = newsletterData[_dt]
+                    for _key in allDatas:
+                        v = allDatas[_key]
+                        if _key not in data:
+                            data[_key] = 0
+                        data[_key] += v
+            totalData.append([_dt, data])
+    else:
+        contactStatus = getContactStatus(conditions, historyMode)
+        completed = sortByDatetime(contactStatus)
+        totalData = []
+
+        if historyMode == 'history':
+            for c in completed:
+                _d = countStatus(completed[c], historyMode)
+                data = _d[0]
+                totalData.append([c, data])
+        else:
+            openIds = []
+            openOnSiteIds = []
+            linkIds = []
+            for c in completed:
+                _d = countStatus(completed[c])
+                open = _d[1]['openId']
+                openOnSite = _d[1]['openOnSiteId']
+                link = _d[1]['clickId']
+                
+                openCount = 0
+                openOnSiteCount = 0
+                linkCount = 0
+                
+                for id in open:
+                    if not id in openIds:
+                        if not id in openOnSiteIds:
+                            if not id in linkIds:
+                                openIds.append(id)
+                                openCount += 1
+                                
+                for id in openOnSite:
                     if not id in openOnSiteIds:
-                        if not id in linkIds:
-                            openIds.append(id)
-                            openCount += 1
-                            
-            for id in openOnSite:
-                if not id in openOnSiteIds:
-                    openOnSiteIds.append(id)
-                    openOnSiteCount += 1
-                    
-            for id in link:
-                if not id in linkIds:
-                    linkIds.append(id)
-                    linkCount += 1
-                    
-            data = _d[0]
-            data['open'] = openCount
-            data['openOnSite'] = openOnSiteCount
-            data['click'] = linkCount
-            totalData.append([c, data])
+                        openOnSiteIds.append(id)
+                        openOnSiteCount += 1
+                        
+                for id in link:
+                    if not id in linkIds:
+                        linkIds.append(id)
+                        linkCount += 1
+                        
+                data = _d[0]
+                data['open'] = openCount
+                data['openOnSite'] = openOnSiteCount
+                data['click'] = linkCount
+                totalData.append([c, data])
 
     allDates = sorted(dict(totalData).keys())
     for day in allDataDates._days:        
         if day not in allDates:
             totalData.append([day, getEmptyDataStructure()])
+    totalData = sorted(totalData)
 
     allContent = {}
     if dataType == 'number':
@@ -593,7 +663,7 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
                                     status=ContactMailingStatus.SENT
                                 ).count()   
             else:
-                totalSentNewsletter = 0
+                totalSentNewsletter = subscriberCount
             for i in info:
                 if i in ('unsubscribe', 'deliverie', 'error'):
                     totalValue = subscriberCount
@@ -610,7 +680,9 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         if mode == 'single':
             for (day, data) in totalData:
                 result = _do(data, subscriberCount, testSubscriberCount)
-                allContent[day] = result # {'day': (status, value)}
+                allContent[day] = result 
+                # {'day': (status, value)}
+                # {'2017-12-25': [('invaild', 0.0), ('sentTest', 0.0), ...]}
         else:
             accumulativeData = getEmptyDataStructure()
 
